use bevy::{math::NormedVectorSpace, picking::{focus::HoverMap, pointer::PointerId}, prelude::*, utils::HashMap, window::PrimaryWindow};

use crate::ui;

pub struct Plugin;
impl bevy::prelude::Plugin for Plugin {
    fn build(&self, app: &mut App) {
        app.add_systems(Startup, setup)
            .add_systems(Update, move_cam)
            .init_resource::<Pointers>();
    }
}

fn setup(mut cmds: Commands, window: Query<&Window>) {
    let zoom = 2./window.single().width().min(window.single().height());
    cmds.spawn((
        Camera2d,
        Transform::from_scale(Vec3::new(zoom, zoom, 1.))
    ));
}

#[derive(Resource, Default)]
struct Pointers(HashMap<PointerId, Vec2>);

fn move_cam(
    mut cam: Query<&mut Transform, With<Camera2d>>,
    map_ui_entity: Query<Entity, With<ui::MapUIComponent>>,
    mouse_buttons: Res<ButtonInput<MouseButton>>,
    touches: Res<Touches>,
    window: Query<&Window, With<PrimaryWindow>>,
    mut pointers: ResMut<Pointers>,
    hover_map: Res<HoverMap>
) {
    let window = window.single();
    let mut cam = cam.single_mut();
    let map_ui_entity = map_ui_entity.single();
    let ps = hover_map.iter().filter_map(|(id, hit_map)| match id {
            PointerId::Mouse => window.cursor_position().map(|p| (mouse_buttons.pressed(MouseButton::Left), p, pointers.0.get(id).map(|p| *p).unwrap_or(p))),
            PointerId::Touch(i) => touches.get_pressed(*i).map(|t| (true, t.position(), t.previous_position())),
            _ => None
        }.map(|(pressed,new_pos, old_pos)| (pressed,new_pos,old_pos,id,hit_map))
    ).collect::<Vec<_>>();
    let pressed_on_map = ps.iter().filter(|p| p.0 && p.4.contains_key(&map_ui_entity)).collect::<Vec<_>>();
    // let mut movement = Vec2::ZERO;
    // for (pressed, new_pos, old_pos, id, hit_map) in pressed_on_map.iter() {
    //     let delta = new_pos - old_pos;
    //     cam.scale += delta.dot(movement)/delta.norm();
    //     movement.x -= delta.x*cam.scale.x/pressed_num as f32;
    //     movement.y += delta.y*cam.scale.y/pressed_num as f32;
    //     pointers.0.insert(**id, *new_pos);
    // }
    
    let old_midpoint = pressed_on_map.iter().fold(Vec2::ZERO, |acc, (_, _, old_pos, _, _)| {
        acc + (old_pos/pressed_on_map.len() as f32)
    });
    let new_midpoint = pressed_on_map.iter().fold(Vec2::ZERO, |acc, (_, new_pos, _, _, _)| {
        acc + (new_pos/pressed_on_map.len() as f32)
    });
    cam.translation.x -= (new_midpoint.x - old_midpoint.x)*cam.scale.x;
    cam.translation.y += (new_midpoint.y - old_midpoint.y)*cam.scale.y;
    
    if pressed_on_map.len() > 1 {
        let old_d_to_midpoint = pressed_on_map.iter().fold(0., |acc, (_, _, old_pos, _, _)| {
            acc + (old_midpoint-old_pos).norm()
        });
        let new_d_to_midpoint = pressed_on_map.iter().fold(0., |acc, (_, new_pos, _, _, _)| {
            acc + (new_midpoint-new_pos).norm()
        });
        let zoom = new_d_to_midpoint/old_d_to_midpoint;
        dbg!(zoom);
        cam.scale.x /= zoom;
        cam.scale.y /= zoom;
    }

    for (_, new_pos, _, id, _) in ps {
        pointers.0.insert(*id, new_pos);
    }
}